using StudiApp.Models;

namespace StudiApp.Test.Unit.ModelTests;

[TestFixture]
public class ToDoTest
{
	private static readonly Course Course = new()
	{
		Id = 10,
		Name = "Test"
	};

	private readonly ToDo _toDo = new()
	{
		Id = 1,
		Name = "ToDo1",
		Description = "Description1",
		Course = Course,
		CourseId = Course.Id,
		Due = new DateTime(2023, 05, 22, 10, 11, 25),
		IsCompleted = false
	};

	[Test]
	public void Equals_SameProperties_ReturnsTrue()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.True);
	}

	[Test]
	public void Equals_DifferentId_ReturnsFalse()
	{
		var expectedToDo = new ToDo
		{
			Id = 99,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_DifferentName_ReturnsFalse()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo99",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_DifferentDescription_ReturnsFalse()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description99",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_DifferentCourseId_ReturnsFalse()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = 99,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_DifferentDue_ReturnsFalse()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2099, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_DifferentIsCompleted_ReturnsFalse()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = true
		};

		var result = _toDo.Equals(expectedToDo);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_WrongType_ReturnsFalse()
	{
		var expected = new Mark();

		var result = _toDo.Equals(expected);

		Assert.That(result, Is.False);
	}

	[Test]
	public void Equals_Null_ReturnsFalse()
	{
		var result = _toDo.Equals(null);

		Assert.That(result, Is.False);
	}

	[Test]
	public void GetHashcode_SameProperties_ReturnsSameHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DifferentId_ReturnsDifferentHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 99,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.Not.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DifferentName_ReturnsDifferentHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo99",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.Not.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_NameIsNullOnBothToDos_ReturnsSameHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = null,
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};
		var actualToDo = new ToDo
		{
			Id = 1,
			Name = null,
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(actualToDo.GetHashCode(), Is.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DifferentDescription_ReturnsDifferentHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description99",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.Not.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DescriptionIsNullOnBothToDos_ReturnsSameHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = null,
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};
		var actualToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = null,
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(actualToDo.GetHashCode(), Is.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DifferentCourseId_ReturnsDifferentHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = 9999,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.Not.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DifferentDue_ReturnsDifferentHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2099, 05, 22, 10, 11, 25),
			IsCompleted = false
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.Not.EqualTo(expected));
	}

	[Test]
	public void GetHashcode_DifferentIsCompleted_ReturnsDifferentHashcode()
	{
		var expectedToDo = new ToDo
		{
			Id = 1,
			Name = "ToDo1",
			Description = "Description1",
			Course = Course,
			CourseId = Course.Id,
			Due = new DateTime(2023, 05, 22, 10, 11, 25),
			IsCompleted = true
		};

		var expected = expectedToDo.GetHashCode();

		Assert.That(_toDo.GetHashCode(), Is.Not.EqualTo(expected));
	}
}