﻿using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator;
using StudiApp.Services.InputValidator.Strategy;

namespace StudiApp.Test.Unit.ServicesTests.InputValidatorTests;

[TestFixture]
public class MarkValidatorTest
{
	[SetUp]
	public void SetUp()
	{
		_testMark = CreateValidMark();
		_testMark.Course = CreateValidCourse();
	}

	private const string AllInvalidErrorMessage =
		"Please choose a Course for the Mark\nMark value out of range, must be between 1.0 and 6.0\nMark Percentage out of range, must be in range 1% up to 100%\nMark Description can't be longer than 250 chars\n";

	private const string CourseNullErrorMessage = "Please choose a Course for the Mark\n";
	private const string MarkValueErrorMessage = "Mark value out of range, must be between 1.0 and 6.0\n";
	private const string MarkPercentageErrorMessage = "Mark Percentage out of range, must be in range 1% up to 100%\n";
	private const string OverallPercentageErrorMessage = "Percentage is above 100% over all marks of this course\n";
	private const string DescriptionErrorMessage = "Mark Description can't be longer than 250 chars\n";
	private const string PercentageTooBigErrorMessage = MarkPercentageErrorMessage + OverallPercentageErrorMessage;

	private readonly IInputValidator _inputValidator = new InputValidator();
	private Mark _testMark = new();

	private static Mark CreateValidMark()
	{
		return new Mark
		{
			Id = 1,
			Value = 5.0,
			Percentage = 0.5
		};
	}

	private static Course CreateValidCourse()
	{
		return new Course
		{
			Marks = new List<Mark>()
		};
	}


	[Test]
	public void TestAllInputValid()
	{
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.True);
			Assert.That(result.ErrorMessage, Is.Null);
		});
	}

	[Test]
	public void TestAllInputInValid()
	{
		_testMark.Course = null;
		_testMark.Value = 0.0;
		_testMark.Percentage = 0;
		_testMark.Description = new string('*', 251);

		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(AllInvalidErrorMessage));
		});
	}

	[Test]
	public void TestMarkCourseNotSet()
	{
		_testMark.Course = null;
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(CourseNullErrorMessage));
		});
	}

	[Test]
	public void TestMarkValueTooBig()
	{
		_testMark.Value = 10.0;
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(MarkValueErrorMessage));
		});
	}

	[Test]
	public void TestMarkValueTooSmall()
	{
		_testMark.Value = 0;
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(MarkValueErrorMessage));
		});
	}

	[Test]
	public void TestMarkPercentageTooSmall()
	{
		_testMark.Percentage = 0;
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(MarkPercentageErrorMessage));
		});
	}

	[Test]
	public void TestMarkPercentageTooBig()
	{
		_testMark.Percentage = 1.1;
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(PercentageTooBigErrorMessage));
		});
	}

	[Test]
	public void TestMarkPercentageOverallAbove100()
	{
		_testMark.Course.Marks.Add(new Mark
		{
			Id = 0,
			Value = 1.0,
			Percentage = 0.51
		});

		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(OverallPercentageErrorMessage));
		});
	}

	[Test]
	public void TestMarkDescriptionTooBig()
	{
		_testMark.Description = new string('*', 251);
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.False);
			Assert.That(result.ErrorMessage, Is.EqualTo(DescriptionErrorMessage));
		});
	}

	[Test]
	public void TestMarkDescriptionNull()
	{
		_testMark.Description = null;
		var result = _inputValidator.IsValid(new MarkValidatorStrategy(_testMark));
		Assert.Multiple(() =>
		{
			Assert.That(result.IsValid, Is.True);
			Assert.That(result.ErrorMessage, Is.Null);
		});
	}
}