using System.Linq.Expressions;
using Moq;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Repository;

namespace StudiApp.Test.Unit.ServicesTests.RepositoryManagerTests;

public class RecursiveRepositoryManagerTests
{
	private Mock<IRepository<Course, int>> _courseRepository = null!;
	private Mock<IRepository<Lecture, int>> _lectureRepository = null!;
	private Mock<IRepository<Mark, int>> _markRepository = null!;
	private RecursiveRepositoryManager _repositoryManager = null!;

	private Mock<IRepository<Semester, int>> _semesterRepository = null!;
	private Mock<IRepository<ToDo, int>> _todoRepository = null!;

	[SetUp]
	public void Setup()
	{
		_semesterRepository = new Mock<IRepository<Semester, int>>();
		_courseRepository = new Mock<IRepository<Course, int>>();
		_markRepository = new Mock<IRepository<Mark, int>>();
		_lectureRepository = new Mock<IRepository<Lecture, int>>();
		_todoRepository = new Mock<IRepository<ToDo, int>>();

		_repositoryManager = new RecursiveRepositoryManager(_semesterRepository.Object, _courseRepository.Object,
			_markRepository.Object, _lectureRepository.Object, _todoRepository.Object);
	}

	#region Helpers

	private static Mark CreateMark(int id, string description, double percentage, double value)
	{
		return new Mark
		{
			Id = id,
			Description = description,
			Percentage = percentage,
			Value = value
		};
	}

	private static void AssertMark(Mark actual, int expectedId, string expectedDescription, double expectedPercentage,
		double expectedValue)
	{
		Assert.Multiple(() =>
		{
			Assert.That(actual.Description, Is.EqualTo(expectedDescription));
			Assert.That(actual.Percentage, Is.EqualTo(expectedPercentage));
			Assert.That(actual.Value, Is.EqualTo(expectedValue));
			Assert.That(actual.Id, Is.EqualTo(expectedId));
		});
	}

	private static Lecture CreateLecture(int id, DayOfWeek day, TimeOnly startTime, TimeOnly endTime, string name,
		string room)
	{
		return new Lecture
		{
			Id = id,
			DayTime = new DayTime
			{
				DayOfWeek = day,
				StartTime = startTime,
				EndTime = endTime
			},
			Name = name,
			Room = room
		};
	}

	private static void AssertLecture(Lecture actual, int expectedId, DayOfWeek expectedDay, TimeOnly expectedStartTime,
		TimeOnly expectedEndTime, string expectedName, string expectedRoom)
	{
		Assert.Multiple(() =>
		{
			Assert.That(actual.Id, Is.EqualTo(expectedId));
			Assert.That(actual.DayTime.DayOfWeek, Is.EqualTo(expectedDay));
			Assert.That(actual.DayTime.StartTime, Is.EqualTo(expectedStartTime));
			Assert.That(actual.DayTime.EndTime, Is.EqualTo(expectedEndTime));
			Assert.That(actual.Name, Is.EqualTo(expectedName));
			Assert.That(actual.Room, Is.EqualTo(expectedRoom));
		});
	}

	private static ToDo CreateTodo(int id, string name, string description, DateTime due, bool isCompleted)
	{
		return new ToDo
		{
			Id = id,
			Name = name,
			Description = description,
			Due = due,
			IsCompleted = isCompleted
		};
	}

	private static void AssertTodo(ToDo actual, int expectedId, string expectedName, string expectedDescription,
		DateTime expectedDue, bool expectedIsCompleted)
	{
		Assert.Multiple(() =>
		{
			Assert.That(actual.Id, Is.EqualTo(expectedId));
			Assert.That(actual.Name, Is.EqualTo(expectedName));
			Assert.That(actual.Description, Is.EqualTo(expectedDescription));
			Assert.That(actual.Due, Is.EqualTo(expectedDue));
			Assert.That(actual.IsCompleted, Is.EqualTo(expectedIsCompleted));
		});
	}

	private static Course CreateCourse(int id, string description, string name, int ects, bool isPassed,
		string lecturerName,
		List<Lecture> lectures, List<Mark> marks, List<ToDo> todos)
	{
		return new Course
		{
			Id = id,
			Description = description,
			Name = name,
			Ects = ects,
			IsPassed = isPassed,
			LecturerName = lecturerName,
			Lectures = lectures,
			Marks = marks,
			ToDos = todos
		};
	}

	private static void AssertCourse(Course actual, int expectedId, string expectedDescription, string expectedName,
		int expectedEcts, bool expectedIsPassed, string expectedLecturerName, IReadOnlyList<Lecture> expectedLectures,
		IReadOnlyList<Mark> expectedMarks, IReadOnlyList<ToDo> expectedTodos)
	{
		Assert.Multiple(() =>
		{
			Assert.That(actual.Id, Is.EqualTo(expectedId));
			Assert.That(actual.Description, Is.EqualTo(expectedDescription));
			Assert.That(actual.Name, Is.EqualTo(expectedName));
			Assert.That(actual.Ects, Is.EqualTo(expectedEcts));
			Assert.That(actual.IsPassed, Is.EqualTo(expectedIsPassed));
			Assert.That(actual.LecturerName, Is.EqualTo(expectedLecturerName));

			Assert.That(actual.Lectures, Has.Count.EqualTo(expectedLectures.Count));
		});

		for (var i = 0; i < expectedLectures.Count; i++)
		{
			var expectedLecture = expectedLectures[i];
			var actualLecture = actual.Lectures[i];

			AssertLecture(actualLecture, expectedLecture.Id, expectedLecture.DayTime.DayOfWeek,
				expectedLecture.DayTime.StartTime, expectedLecture.DayTime.EndTime, expectedLecture.Name,
				expectedLecture.Room);

			Assert.That(actualLecture.Course.Id, Is.EqualTo(expectedId));
		}

		Assert.That(actual.Marks, Has.Count.EqualTo(expectedMarks.Count));
		for (var i = 0; i < expectedMarks.Count; i++)
		{
			var expectedMark = expectedMarks[i];
			var actualMark = actual.Marks[i];

			AssertMark(actualMark, expectedMark.Id, expectedMark.Description, expectedMark.Percentage,
				expectedMark.Value);

			Assert.That(actualMark.Course.Id, Is.EqualTo(expectedId));
		}

		Assert.That(actual.ToDos, Has.Count.EqualTo(expectedTodos.Count));
		for (var i = 0; i < expectedTodos.Count; i++)
		{
			var expectedTodo = expectedTodos[i];
			var actualTodo = actual.ToDos[i];

			AssertTodo(actualTodo, expectedTodo.Id, expectedTodo.Name, expectedTodo.Description, expectedTodo.Due,
				expectedTodo.IsCompleted);
			Assert.That(actualTodo.Course.Id, Is.EqualTo(expectedId));
		}
	}

	private static Semester CreateSemester(int id, string name, DateTime startTime, DateTime endTime,
		List<Course> courses)
	{
		return new Semester
		{
			Id = id, Name = name, DateRange = new DateRange {Start = startTime, End = endTime},
			Courses = courses
		};
	}

	private static void AssertSemester(Semester actual, int expectedId, string expectedName, DateTime expectedStartTime,
		DateTime expectedEndTime, List<Course> expectedCourses)
	{
		Assert.Multiple(() =>
		{
			Assert.That(actual.Id, Is.EqualTo(expectedId));
			Assert.That(actual.Name, Is.EqualTo(expectedName));
			Assert.That(actual.DateRange.Start, Is.EqualTo(expectedStartTime));
			Assert.That(actual.DateRange.End, Is.EqualTo(expectedEndTime));

			Assert.That(actual.Courses, Has.Count.EqualTo(expectedCourses.Count));
		});

		for (var i = 0; i < expectedCourses.Count; i++)
		{
			var expectedCourse = expectedCourses[i];
			var actualCourse = actual.Courses[i];
			AssertCourse(actualCourse, expectedCourse.Id, expectedCourse.Description, expectedCourse.Name,
				expectedCourse.Ects, expectedCourse.IsPassed, expectedCourse.LecturerName, expectedCourse.Lectures,
				expectedCourse.Marks, expectedCourse.ToDos);

			Assert.That(actualCourse.Semester.Id, Is.EqualTo(expectedId));
		}
	}

	#endregion

	#region Mark

	[Test]
	public async Task SaveCreateMarkReturnsCreatedMark()
	{
		// Arrange
		var initialId = 0;
		var description = "This is a mark";
		var percentage = 0.25;
		var value = 5.5;
		var mark = CreateMark(initialId, description, percentage, value);

		_markRepository.Setup(repository => repository.Create(mark)).ReturnsAsync((Mark item) =>
		{
			item.Id = 1;
			return item;
		});

		// Act
		var savedMark = await _repositoryManager.SaveMark(mark);

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(savedMark.Description, Is.EqualTo(description));
			Assert.That(savedMark.Percentage, Is.EqualTo(percentage));
			Assert.That(savedMark.Value, Is.EqualTo(value));
			Assert.That(savedMark.Id, Is.Not.EqualTo(initialId));
		});
	}

	[Test]
	public async Task SaveUpdateMarkReturnsUpdatedMark()
	{
		// Arrange
		var id = 5;
		var description = "TestMark";
		var percentage = 0.5;
		var value = 2.5;
		var mark = CreateMark(id, description, percentage, value);

		_markRepository.Setup(repository => repository.Update(mark)).ReturnsAsync((Mark item) => item);

		// Act
		var updatedMark = await _repositoryManager.SaveMark(mark);

		// Assert
		AssertMark(updatedMark, id, description, percentage, value);
	}

	[Test]
	public async Task GetAllMarksWithNoEntriesReturnsEmptyList()
	{
		// Arrange
		_markRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(() => null);

		// Act
		var marks = await _repositoryManager.GetAllMarks();

		// Assert
		Assert.That(marks, Is.Empty);
	}

	[Test]
	public async Task GetAllMarksWithItemsReturnsItems()
	{
		// Arrange
		var id = 2;
		var description = "TestMark";
		var percentage = 0.5;
		var value = 5;
		var mark = new Mark
		{
			Id = id,
			Description = description,
			Percentage = percentage,
			Value = value
		};

		_markRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(
			() => new List<Mark> {mark});

		// Act
		var receivedMarks = await _repositoryManager.GetAllMarks();

		// Assert
		AssertMark(receivedMarks[0], id, description, percentage, value);
	}

	[Test]
	public async Task GetMarkReturnsMark()
	{
		// Arrange
		var id = 1;
		var description = "TestMark$$$";
		var value = 6;
		var percentage = 0.75;
		var mark = new Mark {Id = id, Description = description, Value = value, Percentage = percentage};

		_markRepository.Setup(repository => repository.Get(mark.Id)).ReturnsAsync(() => mark);

		// Act
		var receivedMark = await _repositoryManager.GetMark(mark.Id);

		// Assert
		AssertMark(receivedMark, id, description, percentage, value);
	}

	[Test]
	public async Task GetMarkReturnsMarkWithParentPropertiesChildrenPropertiesNotNull()
	{
		// Arrange
		var markId = 5;
		var mark = CreateMark(markId, "Testdescription", 1, 2.45);

		var courseId = 2;
		var course = CreateCourse(courseId, "Testtest", "A test course", 0, false, "Lecturer", null!,
			new List<Mark> {mark}, null!);

		_markRepository.Setup(repository => repository.Get(markId)).ReturnsAsync(() =>
		{
			mark.CourseId = courseId;
			return mark;
		});

		_markRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Mark, bool>>>()))
			.ReturnsAsync(new List<Mark> {mark});

		_courseRepository.Setup(repository => repository.Get(courseId)).ReturnsAsync(course);

		_semesterRepository.Setup(repository => repository.Get(0)).ReturnsAsync(() => null!);

		// Act
		var receivedMark = await _repositoryManager.GetMark(markId);

		// Assert
		Assert.That(receivedMark.Course.Marks, Has.Count.EqualTo(1));
	}

	#endregion

	#region Lecture

	[Test]
	public async Task SaveCreateLectureReturnsCreatedLecture()
	{
		// Arrange
		var initialId = 0;
		var day = DayOfWeek.Monday;
		var startTime = new TimeOnly(08, 10);
		var endTime = new TimeOnly(08, 55);
		var name = "TestLecture";
		var room = "1.250";
		var lecture = CreateLecture(initialId, day, startTime, endTime, name, room);

		_lectureRepository.Setup(repository => repository.Create(lecture)).ReturnsAsync(() =>
		{
			lecture.Id = 1;
			return lecture;
		});

		// Act
		var createdLecture = await _repositoryManager.SaveLecture(lecture);

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(lecture.Id, Is.Not.EqualTo(initialId));
			Assert.That(createdLecture.DayTime.DayOfWeek, Is.EqualTo(day));
			Assert.That(createdLecture.DayTime.StartTime, Is.EqualTo(startTime));
			Assert.That(createdLecture.DayTime.EndTime, Is.EqualTo(endTime));
			Assert.That(createdLecture.Name, Is.EqualTo(name));
			Assert.That(createdLecture.Room, Is.EqualTo(room));
		});
	}

	[Test]
	public async Task SaveUpdateLectureReturnsUpdatedLecture()
	{
		// Arrange
		var id = 1;
		var day = DayOfWeek.Wednesday;
		var startTime = new TimeOnly(13, 55);
		var endTime = new TimeOnly(15, 30);
		var name = "Lecture with spaces";
		string room = null!;
		var lecture = CreateLecture(id, day, startTime, endTime, name, room);

		_lectureRepository.Setup(repository => repository.Update(lecture)).ReturnsAsync(lecture);

		// Act
		var savedLecture = await _repositoryManager.SaveLecture(lecture);

		// Assert
		AssertLecture(savedLecture, id, day, startTime, endTime, name, room);
	}

	[Test]
	public async Task GetAllLecturesWithNoEntriesReturnsEmptyList()
	{
		// Arrange
		_lectureRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(() => null);

		// Act
		var lectures = await _repositoryManager.GetAllLectures();

		// Assert
		Assert.That(lectures, Is.Empty);
	}

	[Test]
	public async Task GetAllLecturesWithItemsReturnsItems()
	{
		// Arrange
		var id = 3;
		var day = DayOfWeek.Friday;
		var startTime = new TimeOnly(18, 00);
		var endTime = new TimeOnly(21, 25);
		var name = "Lecture Nr. 27";
		var room = "MyRoom";
		var lecture = CreateLecture(id, day, startTime, endTime, name, room);

		_lectureRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(new List<Lecture> {lecture});

		// Act
		var lectures = await _repositoryManager.GetAllLectures();

		// Assert
		AssertLecture(lectures[0], id, day, startTime, endTime, name, room);
	}

	[Test]
	public async Task GetLectureReturnsLecture()
	{
		// Arrange
		var id = 50068;
		var day = DayOfWeek.Saturday;
		var startTime = new TimeOnly(23, 59);
		var endTime = new TimeOnly(23, 59);
		var name = "TestLect.$";
		var room = "2323";
		var lecture = CreateLecture(id, day, startTime, endTime, name, room);

		_lectureRepository.Setup(repository => repository.Get(lecture.Id)).ReturnsAsync(lecture);

		// Act
		var receivedLecture = await _repositoryManager.GetLecture(lecture.Id);

		// Assert
		AssertLecture(receivedLecture, id, day, startTime, endTime, name, room);
	}

	[Test]
	public async Task GetLectureReturnsLectureWithParentPropertiesChildrenPropertiesNotNull()
	{
		// Arrange
		var lectureId = 27;
		var lecture = CreateLecture(lectureId, DayOfWeek.Monday, new TimeOnly(22, 00), new TimeOnly(23, 59, 59),
			"TestLecture", "Room 1");

		var courseId = 2;
		var course = CreateCourse(courseId, "Testtest", "A test course", 0, false, "Lecturer",
			new List<Lecture> {lecture}, null!, null!);

		_lectureRepository.Setup(repository => repository.Get(lectureId)).ReturnsAsync(() =>
		{
			lecture.CourseId = courseId;
			return lecture;
		});

		_lectureRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Lecture, bool>>>()))
			.ReturnsAsync(new List<Lecture> {lecture});

		_courseRepository.Setup(repository => repository.Get(courseId)).ReturnsAsync(course);

		_semesterRepository.Setup(repository => repository.Get(0)).ReturnsAsync(() => null!);

		// Act
		var receivedLecture = await _repositoryManager.GetLecture(lectureId);

		// Assert
		Assert.That(receivedLecture.Course.Lectures, Has.Count.EqualTo(1));
	}

	#endregion

	#region Todo

	[Test]
	public async Task SaveCreateTodoReturnsCreatedTodo()
	{
		// Arrange
		var initialId = 0;
		var name = "TestTodo";
		var description = "Just a Todo for testing";
		var due = new DateTime(2027, 05, 03);
		var isCompleted = true;
		var todo = CreateTodo(initialId, name, description, due, isCompleted);

		_todoRepository.Setup(repository => repository.Create(todo)).ReturnsAsync(() =>
		{
			todo.Id = 1;
			return todo;
		});

		// Act
		var createdTodo = await _repositoryManager.SaveToDo(todo);

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(createdTodo.Id, Is.Not.EqualTo(initialId));
			Assert.That(createdTodo.Name, Is.EqualTo(name));
			Assert.That(createdTodo.Description, Is.EqualTo(description));
			Assert.That(createdTodo.Due, Is.EqualTo(due));
			Assert.That(createdTodo.IsCompleted, Is.EqualTo(isCompleted));
		});
	}

	[Test]
	public async Task SaveUpdateTodoReturnsUpdatedTodo()
	{
		// Arrange
		var id = 98;
		var name = "Todo123";
		var description = "My Test Todo";
		var due = new DateTime(2023, 04, 20);
		var isCompleted = false;
		var todo = CreateTodo(id, name, description, due, isCompleted);

		_todoRepository.Setup(repository => repository.Update(todo)).ReturnsAsync(todo);

		// Act
		var updatedTodo = await _repositoryManager.SaveToDo(todo);

		// Assert
		AssertTodo(updatedTodo, id, name, description, due, isCompleted);
	}

	[Test]
	public async Task GetAllTodosWithNoEntriesReturnsEmptyList()
	{
		// Arrange
		_todoRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(() => null);

		// Act
		var todos = await _repositoryManager.GetAllToDos();

		// Assert
		Assert.That(todos, Is.Empty);
	}

	[Test]
	public async Task GetAllTodosWithItemsReturnsItems()
	{
		// Arrange
		var id = 5624;
		var name = " T o d o ";
		var description = "a diffrent one";
		var due = new DateTime(2023, 04, 20);
		var isCompleted = true;
		var todo = CreateTodo(id, name, description, due, isCompleted);

		_todoRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(new List<ToDo> {todo});

		// Act
		var todos = await _repositoryManager.GetAllToDos();

		// Assert
		AssertTodo(todos[0], id, name, description, due, isCompleted);
	}

	[Test]
	public async Task GetTodoReturnsTodo()
	{
		// Arrange
		var id = 256;
		var name = "Todo";
		var description = "TestDescription";
		var due = new DateTime(2023, 04, 22);
		var isCompleted = true;
		var todo = CreateTodo(id, name, description, due, isCompleted);

		_todoRepository.Setup(repository => repository.Get(id)).ReturnsAsync(todo);

		// Act
		var receivedTodo = await _repositoryManager.GetToDo(id);

		// Assert
		AssertTodo(receivedTodo, id, name, description, due, isCompleted);
	}

	[Test]
	public async Task GetTodoReturnsTodoWithParentPropertiesChildrenPropertiesNotNull()
	{
		// Arrange
		var todoId = 5798;
		var todo = CreateTodo(todoId, "TestTodo", "Testdescription", new DateTime(2023, 04, 20), false);

		var courseId = 04561;
		var course = CreateCourse(courseId, "Testtest", "A test course", 0, false, "Lecturer",
			null!, null!, new List<ToDo> {todo});

		_todoRepository.Setup(repository => repository.Get(todoId)).ReturnsAsync(() =>
		{
			todo.CourseId = courseId;
			return todo;
		});

		_todoRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<ToDo, bool>>>()))
			.ReturnsAsync(new List<ToDo> {todo});

		_courseRepository.Setup(repository => repository.Get(courseId)).ReturnsAsync(course);

		_semesterRepository.Setup(repository => repository.Get(0)).ReturnsAsync(() => null!);

		// Act
		var receivedTodo = await _repositoryManager.GetToDo(todoId);

		// Assert
		Assert.That(receivedTodo.Course.ToDos, Has.Count.EqualTo(1));
	}

	#endregion

	#region Course

	[Test]
	public async Task SaveCreateCourseReturnsCreatedCourse()
	{
		// Arrange
		var newId = 1;
		var description = "TestDescription";
		var name = "TestCourse";
		var ects = 4;
		var isPassed = false;
		var lecturerName = "TestLecturer";
		var course = CreateCourse(0, description, name, ects, isPassed, lecturerName, null!, null!, null!);

		_courseRepository.Setup(repository => repository.Create(course)).ReturnsAsync((Course item) =>
		{
			item.Id = newId;
			return item;
		});

		// Act
		var createdCourse = await _repositoryManager.SaveCourse(course);

		// Assert
		AssertCourse(createdCourse, newId, description, name, ects, isPassed, lecturerName,
			new List<Lecture>(), new List<Mark>(), new List<ToDo>());
	}

	[Test]
	public async Task SaveCreateCourseWithChildrenReturnsCreatedCourseWithChildrenRecursive()
	{
		// Arrange
		var id = 0;
		var description = "TestDescription";
		var name = "TestCourse";
		var ects = 4;
		var isPassed = false;
		var lecturerName = "TestLecturer";
		var lecture = CreateLecture(0, DayOfWeek.Monday, new TimeOnly(01, 53),
			new TimeOnly(03, 00), "TestLecture", "1.234");
		var lectures = new List<Lecture> {lecture};
		var mark = CreateMark(0, "TestDescription", 1, 1);
		var marks = new List<Mark> {mark};
		var todo = CreateTodo(0, "TestTodo", "TestD", new DateTime(2023, 04, 22), true);
		var todos = new List<ToDo> {todo};
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, lectures, marks, todos);

		_courseRepository.Setup(repository => repository.Create(course)).ReturnsAsync(course);
		_lectureRepository.Setup(repository => repository.Create(lecture)).ReturnsAsync(lecture);
		_markRepository.Setup(repository => repository.Create(mark)).ReturnsAsync(mark);
		_todoRepository.Setup(repository => repository.Create(todo)).ReturnsAsync(todo);

		// Act
		var createdCourse = await _repositoryManager.SaveCourse(course);

		// Assert
		AssertCourse(createdCourse, id, description, name, ects, isPassed, lecturerName, lectures, marks,
			todos);
	}

	[Test]
	public async Task SaveUpdateCourseReturnsUpdatedCourse()
	{
		// Arrange
		var id = 5;
		var description = "Updated description";
		var name = "new name";
		var ects = 6;
		var isPassed = true;
		var lecturerName = "A new lecturer";
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, null!, null!, null!);

		_courseRepository.Setup(repository => repository.Update(course)).ReturnsAsync(course);

		// Act
		var updatedCourse = await _repositoryManager.SaveCourse(course);

		// Assert
		AssertCourse(updatedCourse, id, description, name, ects, isPassed, lecturerName,
			new List<Lecture>(), new List<Mark>(), new List<ToDo>());
	}

	[Test]
	public async Task SaveUpdateCourseWithChildrenReturnsUpdatedCourseWithChildrenRecursive()
	{
		// Arrange
		var id = 2;
		var description = "My new Course";
		var name = "TestCourse";
		var ects = 10;
		var isPassed = true;
		var lecturerName = "TestLecturer";
		var lecture = CreateLecture(1, DayOfWeek.Monday, new TimeOnly(01, 53),
			new TimeOnly(03, 00), "TestLecture", "1.234");
		var lectures = new List<Lecture> {lecture};
		var mark = CreateMark(6, "TestDescription", 1, 1);
		var marks = new List<Mark> {mark};
		var todo = CreateTodo(8789, "TestTodo", "TestD", new DateTime(2023, 04, 22), true);
		var todos = new List<ToDo> {todo};
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, lectures, marks, todos);

		_courseRepository.Setup(repository => repository.Update(course)).ReturnsAsync(course);
		_lectureRepository.Setup(repository => repository.Update(lecture)).ReturnsAsync(lecture);
		_markRepository.Setup(repository => repository.Update(mark)).ReturnsAsync(mark);
		_todoRepository.Setup(repository => repository.Update(todo)).ReturnsAsync(todo);

		// Act
		var updatedCourse = await _repositoryManager.SaveCourse(course);

		// Assert
		AssertCourse(updatedCourse, id, description, name, ects, isPassed, lecturerName, lectures, marks,
			todos);
	}

	[Test]
	public async Task SaveUpdateCourseCreatesNotExistingChildren()
	{
		// Arrange
		var id = 2;
		var description = "My new Course";
		var name = "TestCourse";
		var ects = 10;
		var isPassed = true;
		var lecturerName = "TestLecturer";
		var lecture = CreateLecture(0, DayOfWeek.Monday, new TimeOnly(01, 53),
			new TimeOnly(03, 00), "TestLecture", "1.234");
		var lectures = new List<Lecture> {lecture};
		var mark = CreateMark(0, "TestDescription", 1, 1);
		var marks = new List<Mark> {mark};
		var todo = CreateTodo(0, "TestTodo", "TestD", new DateTime(2023, 04, 22), true);
		var todos = new List<ToDo> {todo};
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, lectures, marks, todos);

		_courseRepository.Setup(repository => repository.Update(course)).ReturnsAsync(course);
		_lectureRepository.Setup(repository => repository.Create(lecture)).ReturnsAsync(lecture);
		_markRepository.Setup(repository => repository.Create(mark)).ReturnsAsync(mark);
		_todoRepository.Setup(repository => repository.Create(todo)).ReturnsAsync(todo);

		// Act
		var updatedCourse = await _repositoryManager.SaveCourse(course);

		// Assert
		AssertCourse(updatedCourse, id, description, name, ects, isPassed, lecturerName, lectures, marks,
			todos);
	}

	[Test]
	public async Task GetAllCoursesWithNoEntriesReturnsEmptyList()
	{
		// Arrange
		_courseRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(() => null);

		// Act
		var courses = await _repositoryManager.GetAllCourses();

		// Assert
		Assert.That(courses, Is.Empty);
	}

	[Test]
	public async Task GetAllCoursesWithItemsReturnsItems()
	{
		// Arrange
		var id = 5;
		var description = "Updated description";
		var name = "new name";
		var ects = 6;
		var isPassed = true;
		var lecturerName = "A new lecturer";
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, null!, null!, null!);

		_courseRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(new List<Course> {course});

		// Act
		var courses = await _repositoryManager.GetAllCourses();

		// Assert
		Assert.That(courses, Has.Count.EqualTo(1));
		AssertCourse(courses[0], id, description, name, ects, isPassed, lecturerName,
			new List<Lecture>(), new List<Mark>(), new List<ToDo>());
	}

	[Test]
	public async Task GetAllCoursesWithItemsReturnsItemsWithChildrenRecursive()
	{
		// Arrange
		var id = 2;
		var description = "My new Course";
		var name = "TestCourse";
		var ects = 10;
		var isPassed = true;
		var lecturerName = "TestLecturer";
		var lecture = CreateLecture(5, DayOfWeek.Monday, new TimeOnly(01, 53),
			new TimeOnly(03, 00), "TestLecture", "1.234");
		var lectures = new List<Lecture> {lecture};
		var mark = CreateMark(2, "TestDescription", 1, 1);
		var marks = new List<Mark> {mark};
		var todo = CreateTodo(7, "TestTodo", "TestD", new DateTime(2023, 04, 22), true);
		var todos = new List<ToDo> {todo};
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, lectures, marks, todos);

		_courseRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(new List<Course> {course});
		_lectureRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Lecture, bool>>>()))
			.ReturnsAsync(
				lectures);
		_markRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Mark, bool>>>()))
			.ReturnsAsync(marks);
		_todoRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<ToDo, bool>>>()))
			.ReturnsAsync(todos);

		// Act
		var courses = await _repositoryManager.GetAllCourses();

		// Assert
		Assert.That(courses, Has.Count.EqualTo(1));
		AssertCourse(courses[0], id, description, name, ects, isPassed, lecturerName, lectures, marks,
			todos);
	}

	[Test]
	public async Task GetCourseReturnsCourse()
	{
		// Arrange
		var id = 69;
		var description = "My course description";
		var name = "CourseName";
		var ects = 2;
		var isPassed = false;
		var lecturerName = "Mr. Lecturer";
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, null!, null!, null!);

		_courseRepository.Setup(repository => repository.Get(id)).ReturnsAsync(course);

		// Act
		var receivedCourse = await _repositoryManager.GetCourse(id);

		// Assert
		AssertCourse(receivedCourse, id, description, name, ects, isPassed, lecturerName,
			new List<Lecture>(), new List<Mark>(), new List<ToDo>());
	}

	[Test]
	public async Task GetCourseWithChildrenReturnsCourseWithChildrenRecursive()
	{
		// Arrange
		var id = 54;
		var description = "TestDescription";
		var name = "TestCourse";
		var ects = 3;
		var isPassed = true;
		var lecturerName = "TestLecturer";
		var lecture = CreateLecture(5, DayOfWeek.Monday, new TimeOnly(01, 53),
			new TimeOnly(03, 00), "TestLecture", "1.234");
		var lectures = new List<Lecture> {lecture};
		var mark = CreateMark(2, "TestDescription", 1, 1);
		var marks = new List<Mark> {mark};
		var todo = CreateTodo(7, "TestTodo", "TestD", new DateTime(2023, 04, 22), true);
		var todos = new List<ToDo> {todo};
		var course = CreateCourse(id, description, name, ects, isPassed, lecturerName, lectures, marks, todos);

		_courseRepository.Setup(repository => repository.Get(id)).ReturnsAsync(course);
		_lectureRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Lecture, bool>>>()))
			.ReturnsAsync(
				lectures);
		_markRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Mark, bool>>>()))
			.ReturnsAsync(marks);
		_todoRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<ToDo, bool>>>()))
			.ReturnsAsync(todos);

		// Act
		var receivedCourse = await _repositoryManager.GetCourse(id);

		// Assert
		AssertCourse(receivedCourse, id, description, name, ects, isPassed, lecturerName, lectures, marks,
			todos);
	}

	[Test]
	public async Task GetCourseReturnsTodoWithParentPropertiesChildrenPropertiesNotNull()
	{
		// Arrange
		var courseId = 123654;
		var course = CreateCourse(courseId, "TestDescription", "TestCourse", 25, false, "TestLecturer", null!, null!,
			null!);

		var semesterId = 999;
		var semester = CreateSemester(semesterId, "TestSemester", new DateTime(), new DateTime(), null!);

		_courseRepository.Setup(repository => repository.Get(courseId)).ReturnsAsync(() =>
		{
			course.SemesterId = semesterId;
			return course;
		});

		_courseRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Course, bool>>>()))
			.ReturnsAsync(new List<Course> {course});

		_semesterRepository.Setup(repository => repository.Get(semesterId)).ReturnsAsync(semester);

		// Act
		var receivedCourse = await _repositoryManager.GetCourse(courseId);

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(receivedCourse.Semester.Courses, Has.Count.EqualTo(1));
			Assert.That(receivedCourse.Semester.DateRange, Is.Not.Null);
			Assert.That(receivedCourse.Semester.Courses[0].Id, Is.EqualTo(courseId));
		});
	}

	#endregion

	#region Semester

	[Test]
	public async Task SaveCreateSemesterReturnsCreatedSemester()
	{
		// Arrange
		var newId = 1;
		var name = "TestCourse";
		var startTime = new DateTime(2023, 03, 01);
		var endTime = new DateTime(2023, 07, 31);
		var semester = CreateSemester(0, name, startTime, endTime, null!);

		_semesterRepository.Setup(repository => repository.Create(semester)).ReturnsAsync((Semester item) =>
		{
			item.Id = newId;
			return item;
		});

		// Act
		var createdSemester = await _repositoryManager.SaveSemester(semester);

		// Assert
		AssertSemester(createdSemester, newId, name, startTime, endTime, new List<Course>());
	}

	[Test]
	public async Task SaveCreateSemesterWithChildrenReturnsCreatedSemesterWithChildrenRecursive()
	{
		// Arrange
		var newId = 1;
		var name = "TestCourse";
		var startTime = new DateTime(2023, 03, 01);
		var endTime = new DateTime(2023, 07, 31);
		var course = CreateCourse(0, "TestDescription", "TestCourse", 20, false,
			"TestLecturer", null!, null!, null!);
		var courses = new List<Course> {course};

		var semester = CreateSemester(0, name, startTime, endTime, courses);

		_semesterRepository.Setup(repository => repository.Create(semester)).ReturnsAsync((Semester item) =>
		{
			item.Id = newId;
			return item;
		});
		_courseRepository.Setup(repository => repository.Create(course)).ReturnsAsync(course);

		// Act
		var createdSemester = await _repositoryManager.SaveSemester(semester);

		// Assert
		AssertSemester(createdSemester, newId, name, startTime, endTime, courses);
	}

	[Test]
	public async Task SaveUpdateSemesterReturnsUpdatedSemester()
	{
		// Arrange
		var id = 9822;
		var name = "TestCourse";
		var startTime = new DateTime(1970, 01, 01);
		var endTime = new DateTime(2105, 12, 31);
		var semester = CreateSemester(id, name, startTime, endTime, null!);

		_semesterRepository.Setup(repository => repository.Update(semester)).ReturnsAsync(semester);

		// Act
		var updatedSemester = await _repositoryManager.SaveSemester(semester);

		// Assert
		AssertSemester(updatedSemester, id, name, startTime, endTime, new List<Course>());
	}

	[Test]
	public async Task SaveUpdateSemesterWithChildrenReturnsUpdatedSemesterWithChildrenRecursive()
	{
		// Arrange
		var id = 5;
		var name = "TestCourse";
		var startTime = new DateTime(2023, 03, 01);
		var endTime = new DateTime(2023, 07, 31);
		var course = CreateCourse(256, "TestDescription", "TestCourse", 20, false,
			"TestLecturer", null!, null!, null!);
		var courses = new List<Course> {course};

		var semester = CreateSemester(id, name, startTime, endTime, courses);

		_semesterRepository.Setup(repository => repository.Update(It.IsAny<Semester>()))
			.ReturnsAsync((Semester item) => item);
		_courseRepository.Setup(repository => repository.Update(It.IsAny<Course>()))
			.ReturnsAsync((Course item) => item);

		// Act
		var updatedSemester = await _repositoryManager.SaveSemester(semester);

		// Assert
		AssertSemester(updatedSemester, id, name, startTime, endTime, courses);
	}

	[Test]
	public async Task SaveUpdateSemesterCreatesNotExistingChildren()
	{
		// Arrange
		var id = 96;
		var name = "TestCourse";
		var startTime = new DateTime(2023, 03, 01);
		var endTime = new DateTime(2023, 07, 31);
		var course = CreateCourse(0, "TestDescription", "TestCourse", 20, false,
			"TestLecturer", null!, null!, null!);
		var courses = new List<Course> {course};

		var semester = CreateSemester(id, name, startTime, endTime, courses);

		_semesterRepository.Setup(repository => repository.Update(It.IsAny<Semester>()))
			.ReturnsAsync((Semester item) => item);
		_courseRepository.Setup(repository => repository.Create(It.IsAny<Course>()))
			.ReturnsAsync((Course item) => item);

		// Act
		var updatedSemester = await _repositoryManager.SaveSemester(semester);

		// Assert
		AssertSemester(updatedSemester, id, name, startTime, endTime, courses);
	}

	[Test]
	public async Task GetAllSemestersWithNoEntriesReturnsEmptyList()
	{
		// Arrange
		_semesterRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(() => null);

		// Act
		var courses = await _repositoryManager.GetAllSemesters();

		// Assert
		Assert.That(courses, Is.Empty);
	}

	[Test]
	public async Task GetAllSemestersWithItemsReturnsItems()
	{
		// Arrange
		var id = 684252;
		var name = "TestCourse";
		var startTime = new DateTime(2023, 03, 01);
		var endTime = new DateTime(2023, 07, 31);
		var semester = CreateSemester(id, name, startTime, endTime, null!);

		_semesterRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(new List<Semester> {semester});

		// Act
		var semesters = await _repositoryManager.GetAllSemesters();

		// Assert
		Assert.That(semesters, Has.Count.EqualTo(1));
		AssertSemester(semesters[0], id, name, startTime, endTime, new List<Course>());
	}

	[Test]
	public async Task GetAllSemestersWithItemsReturnsItemsWithChildrenRecursive()
	{
		// Arrange
		var id = 420;
		var name = "Course";
		var startTime = new DateTime(2023, 04, 20);
		var endTime = new DateTime(2023, 04, 22);
		var course = CreateCourse(634654, "TestDescription", "TestCourse", 20, false,
			"TestLecturer", null!, null!, null!);
		var courses = new List<Course> {course};

		var semester = CreateSemester(id, name, startTime, endTime, courses);

		_semesterRepository.Setup(repository => repository.GetAll(null)).ReturnsAsync(new List<Semester> {semester});
		_courseRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Course, bool>>>()))
			.ReturnsAsync(new List<Course> {course});

		// Act
		var semesters = await _repositoryManager.GetAllSemesters();

		// Assert
		Assert.That(semesters, Has.Count.EqualTo(1));
		AssertSemester(semesters[0], id, name, startTime, endTime, courses);
	}

	[Test]
	public async Task GetSemesterReturnsSemester()
	{
		// Arrange
		var id = 256128;
		var name = "A Course";
		var startTime = new DateTime(2100, 01, 01);
		var endTime = new DateTime(2100, 07, 31);
		var semester = CreateSemester(id, name, startTime, endTime, null!);

		_semesterRepository.Setup(repository => repository.Get(id)).ReturnsAsync(semester);

		// Act
		var receivedSemester = await _repositoryManager.GetSemester(id);

		// Assert
		AssertSemester(receivedSemester, id, name, startTime, endTime, new List<Course>());
	}

	[Test]
	public async Task GetSemesterWithChildrenReturnsSemesterWithChildrenRecursive()
	{
		// Arrange
		var id = 420;
		var name = "Course";
		var startTime = new DateTime(2023, 04, 20);
		var endTime = new DateTime(2023, 04, 22);
		var course = CreateCourse(634654, "TestDescription", "TestCourse", 20, false,
			"TestLecturer", null!, null!, null!);
		var courses = new List<Course> {course};

		var semester = CreateSemester(id, name, startTime, endTime, courses);

		_semesterRepository.Setup(repository => repository.Get(id)).ReturnsAsync(semester);
		_courseRepository.Setup(repository => repository.GetAll(It.IsAny<Expression<Func<Course, bool>>>()))
			.ReturnsAsync(courses);

		// Act
		var receivedSemester = await _repositoryManager.GetSemester(id);

		// Assert
		AssertSemester(receivedSemester, id, name, startTime, endTime, courses);
	}

	#endregion
}