﻿using Microsoft.Extensions.Logging;
using Moq;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Converters;
using StudiApp.Services.ViewModelService;

namespace StudiApp.Test.Unit.ViewModelsTests.DashboardModule;

internal class DashboardModuleTestHelper
{
	internal Course CourseActive;
	internal Course CourseNotActive;
	internal Lecture LectureInNextUp;
	internal NextUpItem LectureNextUpItem;
	internal Lecture LectureNotInNextUp;
	internal Mock<ILogger<ViewModelService>> Logger;
	internal Mock<INavigationManager> NavigationManager;
	internal Mock<IRepositoryManager> RepositoryManager;
	internal Course SecondCourseActive;
	internal Semester SemesterActive;
	internal Semester SemesterNotActive;
	internal ToDo TodoInNextUp;
	internal NextUpItem TodoNextUpItem;
	internal ToDo TodoNotInNextUp;
	internal Mock<IViewModelService> ViewModelService;

	internal DashboardModuleTestHelper()
	{
		//Setup Lectures
		LectureInNextUp = new Lecture
		{
			Name = "LectureInNextUp",
			DayTime = new DayTime
			{
				DayOfWeek = DayConverter.GetNextDayOfWeek(DateTime.Now.DayOfWeek),
				StartTime = TimeOnly.FromDateTime(DateTime.Now.AddMinutes(-1)),
				EndTime = TimeOnly.FromDateTime(DateTime.Now)
			},
			Id = 1
		};

		LectureNotInNextUp = new Lecture
		{
			Name = "LectureNotInNextUp",
			DayTime = new DayTime
			{
				// Add two days that Lecture is not in NextUp
				DayOfWeek = DayConverter.GetNextDayOfWeek(DayConverter.GetNextDayOfWeek(DateTime.Now.DayOfWeek))
			},
			Id = 2
		};

		//Setup Courses
		CourseActive = new Course
		{
			Name = "CourseActive",
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 1,
			Lectures = new List<Lecture> {LectureInNextUp, LectureNotInNextUp}
		};

		SecondCourseActive = new Course
		{
			Name = "SecondCourseActive",
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 2,
			Lectures = new List<Lecture>()
		};

		CourseNotActive = new Course
		{
			Name = "CourseNotActive",
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 3
		};

		//Setup Semesters
		//Add and subtracts 100 days from Datetime.Now (Is always active)
		SemesterActive = new Semester
		{
			Name = "Active",
			DateRange = new DateRange
			{
				Start = DateTime.Now.AddDays(-100),
				End = DateTime.Now.AddDays(100)
			},
			Courses = new List<Course> {CourseActive, SecondCourseActive},
			Id = 1
		};

		SemesterNotActive = new Semester
		{
			Name = "Inactive",
			DateRange = new DateRange
			{
				Start = new DateTime(1999, 01, 01),
				End = new DateTime(2000, 01, 01)
			},
			Courses = new List<Course> {CourseNotActive},
			Id = 2
		};

		var allSemesters = new List<Semester> {SemesterNotActive, SemesterActive};

		//Setup ToDos
		TodoInNextUp = new ToDo
		{
			Name = "TodoInNextUp",
			IsCompleted = false,
			Description = "Description",
			Due = DateTime.Now.AddHours(23.5),
			CourseId = 1,
			Course = CourseActive,
			Id = 1
		};

		TodoNotInNextUp = new ToDo
		{
			Name = "TodoNotInNextUp",
			IsCompleted = false,
			Description = "Description",
			Due = DateTime.Now.AddHours(48),
			CourseId = 1,
			Course = CourseActive,
			Id = 2
		};

		var allToDos = new List<ToDo> {TodoInNextUp, TodoNotInNextUp};

		//Setup Mocks
		NavigationManager = new Mock<INavigationManager>();
		RepositoryManager = new Mock<IRepositoryManager>();
		Logger = new Mock<ILogger<ViewModelService>>();

		ViewModelService = new Mock<IViewModelService>();

		//Setup RepositoryManager
		RepositoryManager.Setup(r => r.GetAllSemesters(null)).Returns(Task.FromResult(allSemesters));
		RepositoryManager.Setup(r => r.GetAllToDos(null)).Returns(Task.FromResult(allToDos));

		//Setup ViewModelService
		ViewModelService.Setup(v => v.NavigationManager).Returns(NavigationManager.Object);
		ViewModelService.Setup(v => v.Logger).Returns(Logger.Object);
		ViewModelService.Setup(v => v.RepositoryManager).Returns(RepositoryManager.Object);

		//Setup NextUpItems
		TodoNextUpItem = new NextUpItem(
			1,
			nameof(ToDo),
			room: "",
			title: "",
			day: DayOfWeek.Monday,
			time: TimeOnly.FromDateTime(DateTime.Now),
			timeString: ""
		);

		LectureNextUpItem = new NextUpItem
		(
			1,
			nameof(Lecture),
			room: "",
			title: "",
			day: DayOfWeek.Monday,
			time: TimeOnly.FromDateTime(DateTime.Now),
			timeString: ""
		);
	}
}