﻿using Moq;
using StudiApp.Models;
using StudiApp.Services.InputValidator;
using StudiApp.Services.InputValidator.Strategy;
using StudiApp.ViewModels;

namespace StudiApp.Test.Unit.ViewModelsTests.ToDoModule;

[TestFixture]
public class ToDoEditViewModelTest
{
	[SetUp]
	public void Setup()
	{
		helper = new ToDoModuleTestHelper();
		viewModel = new ToDoEditViewModel(helper.ViewModelService);
	}

	private ToDoModuleTestHelper helper;
	private ToDoEditViewModel viewModel;

	[Test]
	public void OnAppearing_WithIdAndCourse_LoadsCoursesIsEditAndSelectedCourse()
	{
		var toDo = new ToDo {Id = 5, CourseId = 1};
		viewModel.Id = toDo.Id;
		helper.RepositoryManagerMock.Setup(rm => rm.GetToDo(It.IsAny<int>())).Returns(Task.FromResult(toDo));

		viewModel.OnAppearing().Wait();

		Assert.Multiple(() =>
		{
			Assert.That(viewModel.CurrentToDo, Is.EqualTo(toDo));
			Assert.That(viewModel.IsEdit, Is.EqualTo(true));
			CollectionAssert.AreEqual(viewModel.Courses, helper.GetCourses().Result);
			Assert.That(viewModel.SelectedCourse, Is.EqualTo(helper.Course1));
		});
	}

	[Test]
	public void OnAppearing_WithIdNoCourse_LoadsCoursesIsEditAndSelectedCourse()
	{
		var toDo = new ToDo {Id = 5, CourseId = 100};
		viewModel.Id = toDo.Id;
		helper.RepositoryManagerMock.Setup(rm => rm.GetToDo(It.IsAny<int>())).Returns(Task.FromResult(toDo));

		viewModel.OnAppearing().Wait();

		Assert.Multiple(() =>
		{
			Assert.That(viewModel.CurrentToDo, Is.EqualTo(toDo));
			Assert.That(viewModel.IsEdit, Is.EqualTo(true));
			CollectionAssert.AreEqual(viewModel.Courses, helper.GetCourses().Result);
			Assert.That(viewModel.SelectedCourse, Is.EqualTo(null));
		});
	}

	[Test]
	public void Save_SelectedCourseAndCurrentToDo_SetsCorrectParameters()
	{
		var toDo = new ToDo {Id = 5, CourseId = helper.Course2.Id, Course = helper.Course2};
		viewModel.SelectedCourse = helper.Course1;
		viewModel.CurrentToDo = toDo;
		helper.InputValidatorMock.Setup(iv => iv.IsValid(It.IsAny<ToDoValidatorStrategy>()))
			.Returns(new ValidatorItem(true));

		viewModel.Save().Wait();

		Assert.Multiple(() =>
		{
			Assert.That(viewModel.CurrentToDo, Is.EqualTo(toDo));
			Assert.That(viewModel.CurrentToDo.Course, Is.EqualTo(helper.Course1));
			Assert.That(viewModel.CurrentToDo.CourseId, Is.EqualTo(helper.Course1.Id));
			Assert.That(viewModel.SelectedCourse, Is.EqualTo(helper.Course1));
			helper.RepositoryManagerMock.Verify(x => x.SaveToDo(It.Is<ToDo>(t => t == toDo)), Times.Once);
			helper.NavigationManagerMock.Verify(x => x.NavigateBack(), Times.Once);
		});
	}

	[Test]
	public void Save_NoSelectedCourse_SetsCourseToNullAndCourseIdToZero()
	{
		var toDo = new ToDo {Id = 5, CourseId = helper.Course1.Id, Course = helper.Course1};
		viewModel.CurrentToDo = toDo;
		helper.InputValidatorMock.Setup(iv => iv.IsValid(It.IsAny<ToDoValidatorStrategy>()))
			.Returns(new ValidatorItem(true));

		viewModel.Save().Wait();

		Assert.Multiple(() =>
		{
			Assert.That(viewModel.CurrentToDo, Is.EqualTo(toDo));
			Assert.That(viewModel.CurrentToDo.Course, Is.Null);
			Assert.That(viewModel.CurrentToDo.CourseId, Is.EqualTo(0));
			Assert.That(viewModel.SelectedCourse, Is.EqualTo(null));
			helper.RepositoryManagerMock.Verify(x => x.SaveToDo(It.Is<ToDo>(t => t == toDo)), Times.Once);
			helper.NavigationManagerMock.Verify(x => x.NavigateBack(), Times.Once);
		});
	}

	[Test]
	public void Save_NoNameGiven_ReturnsErrorMessageß()
	{
		var toDo = new ToDo {Id = 5, CourseId = helper.Course1.Id, Course = helper.Course1};
		viewModel.CurrentToDo = toDo;
		viewModel.SelectedCourse = helper.Course1;
		helper.InputValidatorMock.Setup(iv => iv.IsValid(It.IsAny<ToDoValidatorStrategy>()))
			.Returns(new ValidatorItem(false, "ErrorMessage"));

		viewModel.Save().Wait();

		Assert.Multiple(() =>
		{
			Assert.That(viewModel.CurrentToDo, Is.EqualTo(toDo));
			Assert.That(viewModel.CurrentToDo.Course, Is.EqualTo(helper.Course1));
			Assert.That(viewModel.CurrentToDo.CourseId, Is.EqualTo(helper.Course1.Id));
			helper.RepositoryManagerMock.Verify(x => x.SaveToDo(It.IsAny<ToDo>()), Times.Never);
			helper.DialogServiceMock.Verify(x => x.ShowAlertAsync(It.Is<string>(s => s == "ErrorMessage")), Times.Once);
		});
	}

	[Test]
	public void DateAndTime_SetDateAndTime_CurrentToDoIsUpdated()
	{
		var toDo = new ToDo {Id = 5, CourseId = helper.Course1.Id, Course = helper.Course1};
		viewModel.CurrentToDo = toDo;

		var date = new DateTime(2023, 5, 20);
		var time = new TimeSpan(18, 30, 00);
		viewModel.CurrentToDoDate = date;
		viewModel.CurrentToDoTime = time;

		Assert.Multiple(() =>
		{
			Assert.That(
				viewModel.CurrentToDo.Due,
				Is.EqualTo(new DateTime(2023, 05, 20, 18, 30, 00))
			);
			Assert.That(viewModel.CurrentToDoDate, Is.EqualTo(date));
			Assert.That(viewModel.CurrentToDoTime, Is.EqualTo(time));
		});
	}

	[Test]
	public void DateAndTime_NullableHasNoValue_DoesNoChangeValue()
	{
		var dateTime = new DateTime(2023, 05, 20, 18, 30, 00);
		var toDo = new ToDo
		{
			Id = 5,
			CourseId = helper.Course1.Id,
			Course = helper.Course1,
			Due = new DateTime(2023, 05, 20, 18, 30, 00)
		};
		viewModel.CurrentToDo = toDo;

		viewModel.CurrentToDoDate = null;
		viewModel.CurrentToDoTime = null;

		Assert.That(viewModel.CurrentToDo.Due, Is.EqualTo(dateTime));
	}
}