﻿using System.Collections.ObjectModel;
using Moq;
using StudiApp.Models;
using StudiApp.ViewModels;
using StudiApp.Views;

namespace StudiApp.Test.Unit.ViewModelsTests.ToDoModule;

[TestFixture]
public class ToDoOverviewViewModelTests
{
	[SetUp]
	public void Setup()
	{
		helper = new ToDoModuleTestHelper();
		viewModel = new ToDoOverviewViewModel(helper.ViewModelService);
	}

	private ToDoModuleTestHelper helper;
	private ToDoOverviewViewModel viewModel;

	[Test]
	public void OnAppearing_LoadsToDosAndSetsTitle()
	{
		viewModel.IsShowArchiveSet = false;
		viewModel.Title = "StartTitle";

		viewModel.OnAppearing().Wait();

		var expected = new ObservableCollection<ToDo> {helper.Todo1};
		Assert.Multiple(() =>
		{
			Assert.That(viewModel.Title, Is.EqualTo("Pending ToDo's"));
			CollectionAssert.AreEqual(expected, viewModel.ToDos);
		});
	}

	[Test]
	public void ShowArchivedClicked_LoadsToDosAndSetsTitle()
	{
		viewModel.IsShowArchiveSet = true;
		helper.ShowArchived = true;
		viewModel.Title = "StartTitle";

		viewModel.ShowArchivedClicked().Wait();

		var expectedToDo = new ToDo
		{
			Id = 2,
			Course = helper.Course2,
			CourseId = helper.Course2.Id,
			Name = "ToDo2",
			IsCompleted = true
		};
		var expected = new ObservableCollection<ToDo> {expectedToDo};
		Assert.Multiple(() =>
		{
			Assert.That(viewModel.Title, Is.EqualTo("Archived ToDo's"));
			Assert.That(expected, Is.EquivalentTo(viewModel.ToDos));
		});
	}

	[Test]
	public void NewToDoClicked_NavigateToNewToDo()
	{
		viewModel.NewToDoClicked().Wait();

		helper.NavigationManagerMock.Verify(x =>
			x.NavigateTo(nameof(ToDoNewView)), Times.Once());
	}

	[Test]
	public void Edit_NavigateToEditToDo()
	{
		viewModel.Edit(helper.Todo1).Wait();

		Assert.Multiple(() =>
		{
			helper.NavigationManagerMock.Verify(x =>
				x.NavigateTo(nameof(ToDoEditView), "Id", helper.Todo1.Id), Times.Once());
		});
	}

	[Test]
	public void ToggleComplete_SavesToDoAndReloads()
	{
		viewModel.IsShowArchiveSet = false;

		viewModel.ToggleComplete(helper.Todo2).Wait();

		var expectedTodo = new ToDo
		{
			Id = 2,
			Course = helper.Course2,
			CourseId = helper.Course2.Id,
			Name = "ToDo2",
			IsCompleted = false
		};
		var expected = new ObservableCollection<ToDo> {helper.Todo1, expectedTodo};
		Assert.Multiple(() =>
		{
			helper.RepositoryManagerMock.Verify(x => x.SaveToDo(helper.Todo2), Times.Once());
			Assert.That(expectedTodo, Is.EqualTo(helper.Todo2));
			Assert.That(expected, Is.EquivalentTo(viewModel.ToDos));
		});
	}

	[Test]
	public void Delete_PositiveConfirmation_DeletesAndReloads()
	{
		var toDo = helper.Todo1;
		helper.DialogServiceMock.Setup(ds => ds.ShowDeleteConfirmationAsync(
				It.IsAny<string>(),
				It.IsAny<string>()))
			.Returns(Task.FromResult(true));

		viewModel.DeleteToDo(toDo).Wait();

		Assert.Multiple(() =>
		{
			helper.RepositoryManagerMock.Verify(x => x.DeleteToDo(It.Is<ToDo>(t => t == toDo)), Times.Once);
			helper.DialogServiceMock.Verify(x => x.ShowDeleteConfirmationAsync("ToDo", helper.Todo1.Name), Times.Once);
		});
	}

	[Test]
	public void Delete_NegativeConfirmation_NothingHappens()
	{
		var toDo = new ToDo {Id = 5, CourseId = helper.Course1.Id, Course = helper.Course1};
		helper.DialogServiceMock.Setup(ds => ds.ShowDeleteConfirmationAsync(
				It.IsAny<string>(),
				It.IsAny<string>()))
			.Returns(Task.FromResult(false));

		viewModel.DeleteToDo(toDo).Wait();

		Assert.Multiple(() =>
		{
			helper.RepositoryManagerMock.Verify(x => x.DeleteToDo(It.Is<ToDo>(t => t == toDo)), Times.Never);
			helper.DialogServiceMock.Verify(x => x.ShowDeleteConfirmationAsync("ToDo", toDo.Name), Times.Once);
		});
	}
}