using System.Linq.Expressions;
using StudiApp.Interfaces;
using StudiApp.Models;

namespace StudiApp.Services.Repository;

public class RecursiveRepositoryManager : IRepositoryManager
{
	private readonly IRepository<Course, int> _courseRepository;
	private readonly IRepository<Lecture, int> _lectureRepository;
	private readonly IRepository<Mark, int> _markRepository;
	private readonly IRepository<Semester, int> _semesterRepository;
	private readonly IRepository<ToDo, int> _todoRepository;

	public RecursiveRepositoryManager(
		IRepository<Semester, int> semesterRepository,
		IRepository<Course, int> courseRepository,
		IRepository<Mark, int> markRepository,
		IRepository<Lecture, int> lectureRepository,
		IRepository<ToDo, int> todoRepository)
	{
		_semesterRepository = semesterRepository;
		_courseRepository = courseRepository;
		_markRepository = markRepository;
		_lectureRepository = lectureRepository;
		_todoRepository = todoRepository;
	}

	private static async Task<T> SaveItem<T>(T item, IRepository<T, int> repository)
		where T : IRepositoryItem<int>
	{
		if (item.Id == 0) return await repository.Create(item);

		return await repository.Update(item);
	}

	private static async Task DeleteChildren<T>(List<T> items, Func<T, Task> deleteFunc)
		where T : IRepositoryItem<int>
	{
		if (items is not null)
			foreach (var item in items)
				await deleteFunc(item);
	}

	private async Task<T> AssignParentCourseRecursive<T>(T item)
		where T : class, ICourseMember
	{
		if (item is null) return null;

		item.Course = await GetCourse(item.CourseId);
		return item;
	}

	private async Task<T> AssignParentSemesterRecursive<T>(T item)
		where T : class, ISemesterMember
	{
		if (item is null) return null;

		item.Semester = await GetSemester(item.SemesterId);
		return item;
	}

	#region Mark

	public async Task<Mark> SaveMark(Mark mark)
	{
		return await SaveItem(mark, _markRepository);
	}

	public async Task<List<Mark>> GetAllMarks(Expression<Func<Mark, bool>> query = null)
	{
		return await _markRepository.GetAll(query) ?? new List<Mark>();
	}

	public async Task<Mark> GetMark(int id)
	{
		var mark = await _markRepository.Get(id);
		mark = await AssignParentCourseRecursive(mark);
		return mark;
	}

	public async Task DeleteMark(Mark mark)
	{
		await _markRepository.Delete(mark);
	}

	#endregion

	#region ToDo

	public async Task<ToDo> SaveToDo(ToDo todo)
	{
		return await SaveItem(todo, _todoRepository);
	}

	public async Task<List<ToDo>> GetAllToDos(Expression<Func<ToDo, bool>> query = null)
	{
		return await _todoRepository.GetAll(query) ?? new List<ToDo>();
	}

	public async Task<ToDo> GetToDo(int id)
	{
		var todo = await _todoRepository.Get(id);
		todo = await AssignParentCourseRecursive(todo);
		return todo;
	}

	public async Task DeleteToDo(ToDo todo)
	{
		await _todoRepository.Delete(todo);
	}

	#endregion

	#region Lecture

	public async Task<Lecture> SaveLecture(Lecture lecture)
	{
		return await SaveItem(lecture, _lectureRepository);
	}

	public async Task<List<Lecture>> GetAllLectures(Expression<Func<Lecture, bool>> query = null)
	{
		return await _lectureRepository.GetAll(query) ?? new List<Lecture>();
	}

	public async Task<Lecture> GetLecture(int id)
	{
		var lecture = await _lectureRepository.Get(id);
		lecture = await AssignParentCourseRecursive(lecture);
		return lecture;
	}

	public async Task DeleteLecture(Lecture lecture)
	{
		await _lectureRepository.Delete(lecture);
	}

	#endregion

	#region Course

	public async Task<Course> SaveCourse(Course course)
	{
		var savedCourse = await SaveItem(course, _courseRepository);

		savedCourse.Marks = await SaveCourseItems(savedCourse, course.Marks, SaveMark);
		savedCourse.ToDos = await SaveCourseItems(savedCourse, course.ToDos, SaveToDo);
		savedCourse.Lectures = await SaveCourseItems(savedCourse, course.Lectures, SaveLecture);

		return savedCourse;
	}

	public async Task<List<Course>> GetAllCourses(Expression<Func<Course, bool>> query = null)
	{
		var courses = await _courseRepository.GetAll(query) ?? new List<Course>();

		foreach (var course in courses)
		{
			course.Marks = await GetCourseItems<Mark>(course, GetAllMarks);
			course.ToDos = await GetCourseItems<ToDo>(course, GetAllToDos);
			course.Lectures = await GetCourseItems<Lecture>(course, GetAllLectures);
		}

		return courses;
	}

	public async Task<Course> GetCourse(int id)
	{
		var course = await _courseRepository.Get(id);
		if (course is null) return null;

		course.Marks = await GetCourseItems<Mark>(course, GetAllMarks);
		course.ToDos = await GetCourseItems<ToDo>(course, GetAllToDos);
		course.Lectures = await GetCourseItems<Lecture>(course, GetAllLectures);

		course = await AssignParentSemesterRecursive(course);

		return course;
	}

	public async Task DeleteCourse(Course course)
	{
		await DeleteChildren(course.Marks, DeleteMark);
		await DeleteChildren(course.ToDos, DeleteToDo);
		await DeleteChildren(course.Lectures, DeleteLecture);

		await _courseRepository.Delete(course);
	}

	private static async Task<List<T>> SaveCourseItems<T>(Course parentCourse, List<T> items, Func<T, Task<T>> saveFunc)
		where T : class, ICourseMember, IRepositoryItem<int>
	{
		var savedItems = new List<T>();

		if (items is not null)
			foreach (var item in items)
			{
				item.CourseId = parentCourse.Id;
				item.Course = parentCourse;

				var savedItem = await saveFunc(item);
				savedItems.Add(savedItem);
			}

		return savedItems;
	}

	private static async Task<List<T>> GetCourseItems<T>(Course parentCourse,
		Func<Expression<Func<T, bool>>, Task<List<T>>> getFunc)
		where T : class, ICourseMember, IRepositoryItem<int>
	{
		var items = await getFunc(item => item.CourseId == parentCourse.Id);
		foreach (var item in items)
		{
			item.CourseId = parentCourse.Id;
			item.Course = parentCourse;
		}

		return items;
	}

	#endregion

	#region Semester

	public async Task<Semester> SaveSemester(Semester semester)
	{
		var savedSemester = await SaveItem(semester, _semesterRepository);
		savedSemester.Courses = await SaveSemesterItems(savedSemester, semester.Courses, SaveCourse);

		return savedSemester;
	}

	public async Task<List<Semester>> GetAllSemesters(Expression<Func<Semester, bool>> query = null)
	{
		var semesters = await _semesterRepository.GetAll(query) ?? new List<Semester>();

		foreach (var semester in semesters) semester.Courses = await GetSemesterItems<Course>(semester, GetAllCourses);

		return semesters;
	}

	public async Task<Semester> GetSemester(int id)
	{
		var semester = await _semesterRepository.Get(id);
		if (semester is null) return null;

		semester.Courses = await GetSemesterItems<Course>(semester, GetAllCourses);

		return semester;
	}

	public async Task DeleteSemester(Semester semester)
	{
		await DeleteChildren(semester.Courses, DeleteCourse);
		await _semesterRepository.Delete(semester);
	}

	private static async Task<List<T>> SaveSemesterItems<T>(Semester parentSemester, List<T> items,
		Func<T, Task<T>> saveFunc)
		where T : class, ISemesterMember, IRepositoryItem<int>
	{
		var savedItems = new List<T>();

		if (items is not null)
			foreach (var item in items)
			{
				item.SemesterId = parentSemester.Id;
				item.Semester = parentSemester;

				var savedItem = await saveFunc(item);
				savedItems.Add(savedItem);
			}

		return savedItems;
	}

	private static async Task<List<T>> GetSemesterItems<T>(Semester parentSemester,
		Func<Expression<Func<T, bool>>, Task<List<T>>> getFunc)
		where T : class, ISemesterMember, IRepositoryItem<int>
	{
		var items = await getFunc(item => item.SemesterId == parentSemester.Id);
		foreach (var item in items)
		{
			item.SemesterId = parentSemester.Id;
			item.Semester = parentSemester;
		}

		return items;
	}

	#endregion
}