﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator.Strategy;
using Syncfusion.Maui.DataSource.Extensions;

namespace StudiApp.ViewModels;

public abstract partial class CoursesViewModelBase : ObservableObject
{
	protected readonly IViewModelService ViewModelService;

	[ObservableProperty] private Course _course;
	[ObservableProperty] private ObservableCollection<Lecture> _lectures;
	[ObservableProperty] private Semester _selectedSemester;
	[ObservableProperty] private ObservableCollection<Semester> _semesters;
	[ObservableProperty] private ObservableCollection<DayOfWeek> _weekDays;

	protected CoursesViewModelBase(IViewModelService viewModelService)
	{
		ViewModelService = viewModelService;
	}

	private ObservableCollection<DayOfWeek> GetWeekDays()
	{
		ViewModelService.Logger.LogInformation("WeekDays Enum initialized");
		return new ObservableCollection<DayOfWeek>(
			Enum.GetValuesAsUnderlyingType(typeof(DayOfWeek)).ToList<DayOfWeek>());
	}

	private async Task<ObservableCollection<Semester>> InitializeSemesters()
	{
		var semesters = new ObservableCollection<Semester>();

		var allSemesters = await ViewModelService.RepositoryManager.GetAllSemesters();

		foreach (var semester in allSemesters) semesters.Add(semester);

		ViewModelService.Logger.LogInformation("Semesters initialized");
		return semesters;
	}

	protected static Lecture CreateEmptyLecture()
	{
		return new Lecture
		{
			DayTime = new DayTime
			{
				StartTime = TimeOnly.FromDateTime(DateTime.Now),
				EndTime = TimeOnly.FromDateTime(DateTime.Now.AddHours(2)),
				DayOfWeek = DayOfWeek.Monday
			}
		};
	}

	protected async Task OnAppearing()
	{
		WeekDays = GetWeekDays();
		Semesters = await InitializeSemesters();

		if (Semesters.Count > 0 && Course.SemesterId > 0)
			SelectedSemester = Semesters.First(semester => semester.Id.Equals(Course.SemesterId));

		ViewModelService.Logger.LogInformation("{ViewModel} initialized", nameof(CoursesViewModelBase));
	}

	protected void SetLectures(List<Lecture> lectures)
	{
		Lectures = new ObservableCollection<Lecture>(lectures);
	}

	protected void SetCourse(Course course)
	{
		Course = course;
	}

	protected void CombineCourseItems()
	{
		Course.Lectures.Clear();
		Lectures.ForEach(lecture => Course.Lectures.Add(lecture));

		Course.Semester = SelectedSemester;

		ViewModelService.Logger.LogInformation("Combined Lectures and Course {CourseName} together", Course.Name);
	}

	[RelayCommand]
	public async Task Save()
	{
		CombineCourseItems();

		var validatorResult = ViewModelService.InputValidator.IsValid(new CourseValidatorStrategy(Course));

		if (validatorResult.IsValid)
		{
			await ViewModelService.RepositoryManager.SaveCourse(Course);
			ViewModelService.Logger.LogInformation("Course ({CourseName}) saved", Course.Name);
			await ViewModelService.NavigationManager.NavigateBack();
		}
		else
		{
			await ViewModelService.DialogService.ShowAlertAsync(validatorResult.ErrorMessage);
			ViewModelService.Logger.LogInformation(
				"Course ({CourseName}) could not save because of errors: {ErrorMessage}", Course.Name,
				validatorResult.ErrorMessage);
		}
	}

	[RelayCommand]
	public void AddLecture()
	{
		Lectures.Add(CreateEmptyLecture());
		ViewModelService.Logger.LogInformation("Added empty lecture for {CourseName}", Course.Name);
	}

	[RelayCommand]
	public async Task DeleteLecture(Lecture lecture)
	{
		var result =
			await ViewModelService.DialogService.ShowDeleteConfirmationAsync(nameof(Lecture), lecture.Name);

		if (result)
		{
			Lectures.Remove(lecture);
			ViewModelService.Logger.LogInformation("Removed Lecture {LectureName} from Course {CourseName}",
				lecture.Name, Course.Name);
		}
	}
}