﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Converters;
using StudiApp.Views;

namespace StudiApp.ViewModels;

public partial class DashboardViewModel : ObservableObject
{
	private readonly IViewModelService _viewModelService;
	[ObservableProperty] private ObservableCollection<Course> _courses;

	private Semester _currentSemester;
	[ObservableProperty] private ObservableCollection<DoubleCourseItem> _doubleCourseItems;
	[ObservableProperty] private ObservableCollection<Lecture> _lectures;
	[ObservableProperty] private ObservableCollection<NextUpItem> _nextUpItems;

	[ObservableProperty] private ObservableCollection<ToDo> _toDos;

	public DashboardViewModel(IViewModelService viewModelService)
	{
		_viewModelService = viewModelService;
		_toDos = new ObservableCollection<ToDo>();
		_lectures = new ObservableCollection<Lecture>();
	}

	private async Task<Semester> GetCurrentSemester()
	{
		var semesters = await _viewModelService.RepositoryManager.GetAllSemesters();

		return Semester.GetCurrent(semesters);
	}

	private ObservableCollection<Course> GetCurrentSemesterCourses()
	{
		return new ObservableCollection<Course>(_currentSemester.Courses);
	}

	private ObservableCollection<DoubleCourseItem> GetDoubleCourseItems()
	{
		var doubleCourseItems = new ObservableCollection<DoubleCourseItem>();

		var currentCourses = Courses.ToList();

		while (currentCourses.Any())
		{
			var courses = currentCourses.Take(2).ToList();
			var doubleCourseItem = new DoubleCourseItem();

			doubleCourseItem.FirstCourse = courses[0];
			doubleCourseItem.FirstCourseName = courses[0].Name;
			if (courses.Count >= 2)
			{
				doubleCourseItem.SecondCourse = courses[1];
				doubleCourseItem.SecondCourseName = courses[1].Name;
			}

			doubleCourseItems.Add(doubleCourseItem);
			currentCourses.RemoveRange(0, courses.Count);
		}

		return doubleCourseItems;
	}

	private ObservableCollection<Lecture> GetNextLectures()
	{
		var currentDayOfWeek = DateTime.Now.DayOfWeek;
		var nextDayOfWeek = DayConverter.GetNextDayOfWeek(currentDayOfWeek);
		var currentTime = TimeOnly.FromDateTime(DateTime.Now);

		var lectures = new ObservableCollection<Lecture>(Courses
			.SelectMany(c => c.Lectures)
			.Where(l =>
			{
				var lectureDayOfWeek = l.DayTime.DayOfWeek;
				var lectureStartTime = l.DayTime.StartTime;

				return (lectureDayOfWeek == currentDayOfWeek && lectureStartTime >= currentTime) ||
				       (lectureDayOfWeek == nextDayOfWeek && lectureStartTime <= currentTime);
			})
			.ToList());

		return lectures;
	}

	private async Task<ObservableCollection<ToDo>> GetNextToDos()
	{
		var currentTime = DateTime.Now;
		var upToTime = currentTime.AddHours(24);
		var toDos = await _viewModelService.RepositoryManager.GetAllToDos();
		toDos = toDos.FindAll(t =>
			t.Due <= upToTime && t.Due >= currentTime && !t.IsCompleted);

		return new ObservableCollection<ToDo>(toDos);
	}

	private ObservableCollection<NextUpItem> GetNextUpItems()
	{
		var unorderedNextUpItems = new ObservableCollection<NextUpItem>();

		foreach (var lecture in Lectures)
		{
			var nextUpItem = new NextUpItem(
				lecture.CourseId,
				nameof(Lecture),
				lecture.Name,
				lecture.Room,
				lecture.DayTime.DayOfWeek,
				lecture.DayTime.StartTime,
				$"{lecture.DayTime.StartTime} - {lecture.DayTime.EndTime}");
			unorderedNextUpItems.Add(nextUpItem);
		}

		foreach (var todo in ToDos)
		{
			var nextUpItem = new NextUpItem(
				todo.Id,
				nameof(ToDo),
				todo.Name,
				null,
				todo.Due.DayOfWeek,
				TimeOnly.FromDateTime(todo.Due),
				TimeOnly.FromDateTime(todo.Due).ToString());
			unorderedNextUpItems.Add(nextUpItem);
		}

		var orderedNextUpItems = unorderedNextUpItems.OrderBy(item => item.Day).ThenBy(item => item.Time);

		return new ObservableCollection<NextUpItem>(orderedNextUpItems);
	}

	[RelayCommand]
	public async Task OnAppearing()
	{
		_currentSemester = await GetCurrentSemester();

		if (_currentSemester != null)
		{
			Courses = GetCurrentSemesterCourses();

			DoubleCourseItems = GetDoubleCourseItems();

			Lectures = GetNextLectures();
		}
		else
		{
			//Cant use clear as the IsNotEmpty Evaluator does not trigger in xaml
			DoubleCourseItems = new ObservableCollection<DoubleCourseItem>();
			Lectures = new ObservableCollection<Lecture>();
		}

		ToDos = await GetNextToDos();

		NextUpItems = GetNextUpItems();

		_viewModelService.Logger.LogInformation("{ViewModel} initialized", nameof(DashboardViewModel));
	}

	[RelayCommand]
	public async Task TapCourse(Course c)
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(CoursesDetailView), "Id", c.Id);

		_viewModelService.Logger.LogInformation("Navigating to {ViewName} of course {CourseName}",
			nameof(CoursesDetailView), c.Name);
	}

	[RelayCommand]
	public async Task TapNextUpItem(NextUpItem item)
	{
		await _viewModelService.NavigationManager.NavigateTo(item.View, "Id", item.Id);

		_viewModelService.Logger.LogInformation("Navigating to Timetable");
	}
}