﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Converters;
using StudiApp.Services.InputValidator.Strategy;

namespace StudiApp.ViewModels;

[QueryProperty("SelectedSemester", "SelectedSemester")]
public abstract partial class MarksOperationsBaseViewModel : ObservableObject
{
	protected readonly IViewModelService ViewModelService;
	[ObservableProperty] private ObservableCollection<Course> _courses;
	[ObservableProperty] private Mark _mark;
	[ObservableProperty] private string _percentage;

	protected MarksOperationsBaseViewModel(IViewModelService viewModelService)
	{
		ViewModelService = viewModelService;
	}

	public Semester SelectedSemester { get; set; }

	private ObservableCollection<Course> GetSelectedSemesterCourses()
	{
		var semesterCourses = SelectedSemester.Courses;
		return new ObservableCollection<Course>(semesterCourses);
	}

	protected void OnAppearing()
	{
		Courses = GetSelectedSemesterCourses();
		ViewModelService.Logger.LogInformation($"{nameof(MarksOperationsBaseViewModel)} initialized");
	}

	protected void SetMark(Mark mark)
	{
		Mark = mark;
	}

	protected void SetPercentage(string percentage)
	{
		Percentage = percentage;
	}

	[RelayCommand]
	public async Task Save()
	{
		try
		{
			Mark.Percentage = PercentageConverter.ConvertToDouble(Percentage);
			var validatorResult = ViewModelService.InputValidator.IsValid(new MarkValidatorStrategy(Mark));

			if (validatorResult.IsValid)
			{
				await ViewModelService.RepositoryManager.SaveMark(Mark);
				ViewModelService.Logger.LogInformation("Mark saved");
				await ViewModelService.NavigationManager.NavigateBack();
			}
			else
			{
				await ViewModelService.DialogService.ShowAlertAsync(validatorResult.ErrorMessage);
				ViewModelService.Logger.LogInformation("Mark couldn't save because of errors: {ErrorMessage}",
					validatorResult.ErrorMessage);
			}
		}
		catch (ArgumentException e)
		{
			await ViewModelService.DialogService.ShowAlertAsync(e.Message);
		}
	}
}