﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Views;
using Syncfusion.Maui.Scheduler;

namespace StudiApp.ViewModels;

public partial class TimetableViewModel : ObservableObject
{
	private readonly ITimetableItemFacade _timetableItemFacade;

	private readonly IViewModelService _viewModelService;

	[ObservableProperty] private ObservableCollection<TimetableItem> _timetableItems;

	public TimetableViewModel(IViewModelService viewModelService, ITimetableItemFacade timetableItemFacade)
	{
		_viewModelService = viewModelService;
		_timetableItemFacade = timetableItemFacade;

		TimetableItems = new ObservableCollection<TimetableItem>();
	}

	private async Task<Semester> GetCurrentSemester()
	{
		var semesters = await _viewModelService.RepositoryManager.GetAllSemesters();
		return Semester.GetCurrent(semesters);
	}

	private static List<Lecture> GetLectures(Semester semester)
	{
		var lectures = new List<Lecture>();
		foreach (var course in semester.Courses) lectures.AddRange(course.Lectures);

		return lectures;
	}

	[RelayCommand]
	public void OnAppearing()
	{
		GetCurrentSemester().ContinueWith(async semesterTask =>
		{
			var timetableItems = new ObservableCollection<TimetableItem>();
			var semester = semesterTask.Result;

			var lectureTimetableItems =
				_timetableItemFacade.FromLectures(GetLectures(semester), nameof(CoursesDetailView));
			foreach (var timetableItem in lectureTimetableItems) timetableItems.Add(timetableItem);

			var todos = await _viewModelService.RepositoryManager.GetAllToDos(t => !t.IsCompleted);
			var todoTimetableItems = _timetableItemFacade.FromTodos(todos, nameof(ToDoEditView));
			foreach (var timetableItem in todoTimetableItems) timetableItems.Add(timetableItem);

			TimetableItems = timetableItems;
		});
	}

	public async void OnSchedulerTapped(object sender, SchedulerTappedEventArgs e)
	{
		var schedulerAppointment = (TimetableItem) e.Appointments.First();

		var parts = schedulerAppointment.Id.Split("-");

		var id = int.Parse(parts[0]);
		var destinationView = parts[1];

		await OnItemTapped(destinationView, id);
	}

	public async Task OnItemTapped(string destinationView, int id)
	{
		await _viewModelService.NavigationManager.NavigateTo(destinationView, "Id", id);

		_viewModelService.Logger.LogInformation("Navigating to {View}", destinationView);
	}
}