﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Views;

namespace StudiApp.ViewModels;

public partial class ToDoOverviewViewModel : ObservableObject
{
	private const string ArchiveTitleString = "Archived ToDo's";
	private const string ActiveTitleString = "Pending ToDo's";

	private readonly IViewModelService _viewModelService;
	[ObservableProperty] private bool _isShowArchiveSet;

	[ObservableProperty] private string _title;
	[ObservableProperty] private ObservableCollection<ToDo> _toDos = new();

	public ToDoOverviewViewModel(IViewModelService viewModelService)
	{
		_viewModelService = viewModelService;
	}

	public bool IsShowArchiveNotSet => !IsShowArchiveSet;

	private string UpdateTitle()
	{
		return IsShowArchiveSet ? ArchiveTitleString : ActiveTitleString;
	}

	private async Task LoadToDos()
	{
		var toDos = await _viewModelService.RepositoryManager
			.GetAllToDos(t => t.IsCompleted == IsShowArchiveSet);

		_viewModelService.Logger.LogInformation("All ToDos have been loaded from the Database");

		ToDos = new ObservableCollection<ToDo>(toDos.OrderBy(t => t.Due));
		Title = UpdateTitle();
		OnPropertyChanged(nameof(IsShowArchiveNotSet));
	}

	[RelayCommand]
	public async Task ShowArchivedClicked()
	{
		await LoadToDos();

		_viewModelService.Logger.LogInformation("Show {Title}. Show archived courses: {IsShowArchivedSet}", Title,
			IsShowArchiveSet);
	}

	[RelayCommand]
	public async Task NewToDoClicked()
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(ToDoNewView));
		_viewModelService.Logger.LogInformation("Navigating to {ViewName} to create new ToDo", nameof(ToDoNewView));
	}

	[RelayCommand]
	public async Task Edit(ToDo t)
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(ToDoEditView), "Id", t.Id);
		_viewModelService.Logger.LogInformation("Navigating to {ViewName}", nameof(ToDoEditView));
	}

	[RelayCommand]
	public async Task ToggleComplete(ToDo t)
	{
		t.IsCompleted = !t.IsCompleted;
		await _viewModelService.RepositoryManager.SaveToDo(t);
		await LoadToDos();
		_viewModelService.Logger.LogInformation("ToDo ({ToDoName}) saved", t.Name);
	}

	[RelayCommand]
	public async Task DeleteToDo(ToDo toDo)
	{
		var result = await _viewModelService.DialogService.ShowDeleteConfirmationAsync(nameof(ToDo), toDo.Name);


		if (result)
		{
			await _viewModelService.RepositoryManager.DeleteToDo(toDo);
			await LoadToDos();
			_viewModelService.Logger.LogInformation("ToDo {ToDoName} deleted", toDo.Name);
		}
	}

	[RelayCommand]
	public async Task OnAppearing()
	{
		await LoadToDos();

		_viewModelService.Logger.LogInformation("{ViewModel} initialized", nameof(CoursesOverviewViewModel));
	}
}