﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator.Strategy;

namespace StudiApp.ViewModels;

public abstract partial class ToDoViewModelBase : ObservableObject
{
	protected readonly IViewModelService ViewModelService;
	[ObservableProperty] private ObservableCollection<Course> _courses;

	[ObservableProperty] private ToDo _currentToDo;
	[ObservableProperty] private Course _selectedCourse;

	protected ToDoViewModelBase(IViewModelService viewModelService)
	{
		ViewModelService = viewModelService;
	}

	public bool IsEdit => CurrentToDo?.Id != 0;

	public DateTime? CurrentToDoDate
	{
		get => CurrentToDo?.Due.Date;
		set
		{
			if (!value.HasValue) return;
			CurrentToDo.Due = new DateTime(
				value.Value.Year, value.Value.Month, value.Value.Day,
				CurrentToDo.Due.Hour, CurrentToDo.Due.Minute, CurrentToDo.Due.Second);
			OnPropertyChanged();
		}
	}

	public TimeSpan? CurrentToDoTime
	{
		get => CurrentToDo?.Due.TimeOfDay;
		set
		{
			if (!value.HasValue) return;
			CurrentToDo.Due = new DateTime(
				CurrentToDo.Due.Year, CurrentToDo.Due.Month, CurrentToDo.Due.Day,
				value.Value.Hours, value.Value.Minutes, value.Value.Seconds);
			OnPropertyChanged();
		}
	}

	private async Task<ObservableCollection<Course>> InitializeCourses()
	{
		var allCourses = await ViewModelService.RepositoryManager.GetAllCourses();

		var courses = new ObservableCollection<Course>(allCourses);

		ViewModelService.Logger.LogInformation("Courses initialized");
		return courses;
	}

	private void SaveSelectedCourseIntoToDo()
	{
		CurrentToDo.Course = SelectedCourse;
		CurrentToDo.CourseId = SelectedCourse is null ? 0 : SelectedCourse.Id;
	}

	protected async Task OnAppearing(ToDo currentToDo = null)
	{
		CurrentToDo = currentToDo ?? new ToDo {Due = DateTime.Now};
		OnPropertyChanged(nameof(IsEdit));
		OnPropertyChanged(nameof(CurrentToDoDate));
		OnPropertyChanged(nameof(CurrentToDoTime));

		Courses = await InitializeCourses();
		SelectedCourse = Courses.FirstOrDefault(course => course.Id.Equals(CurrentToDo.CourseId), SelectedCourse);

		ViewModelService.Logger.LogInformation("{ViewModel} initialized", nameof(ToDoViewModelBase));
	}

	[RelayCommand]
	public async Task Save()
	{
		SaveSelectedCourseIntoToDo();

		var validatorResult = ViewModelService.InputValidator.IsValid(new ToDoValidatorStrategy(CurrentToDo));

		if (validatorResult.IsValid)
		{
			await ViewModelService.RepositoryManager.SaveToDo(CurrentToDo);
			ViewModelService.Logger.LogInformation("ToDo ({ToDoName}) saved", CurrentToDo.Name);
			await ViewModelService.NavigationManager.NavigateBack();
		}
		else
		{
			await ViewModelService.DialogService.ShowAlertAsync(validatorResult.ErrorMessage);
			ViewModelService.Logger.LogInformation(
				"ToDo ({ToDoName}) could not save because of errors: {ErrorMessage}", CurrentToDo.Name,
				validatorResult.ErrorMessage);
		}
	}
}